unit siFltBtn;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ShellAPI;

type

  TGlyphLayout = (glGlyphLeft, glGlyphRight, glGlyphTop, glGlyphBottom);
  TNumGlyphs = 1..4;
  TButtonOrientation = (boHorizontal, boVertical);

  TsiFlatBtn = class(TGraphicControl)
  private
    { Private declarations }
    FIsDown, FHasMouse: boolean;
    FMyRect, TextBounds: TRect;
    FRadius: integer;
    FGlyphLeft, FGlyphTop: integer;
    FTextFlag: integer;
    FGlyph, BMP, ActiveGlyph: TBitmap;
    FCaption: TCaption;
    FDrawFrame: boolean;
    FFrameColor: TColor;
    FGlyphNums: TNumGlyphs;
    FGlyphLayout: TGlyphLayout;
    FButtonOrientation: TButtonOrientation;
    F3DTopCl, F3DBotCl: TColor;
    FElliptic: boolean;
    procedure DrawRaised;
    procedure DrawUsual;
    procedure DrawLowered;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure SetGlyph(Value: TBitmap);
    procedure SetCaption(Value: TCaption);
    procedure SetDrawFrame(Value: boolean);
    procedure SetFrameColor(Value: TColor);
    function GetNumGlyphs: TNumGlyphs;
    procedure SetNumGlyphs(Value: TNumGlyphs);
    procedure SetGlyphLayout(Value: TGlyphLayout);
    procedure SetButtonOrientation(Value: TButtonOrientation);
    procedure Set3DTopCl(Value: TColor);
    procedure Set3DBotCl(Value: TColor);
    procedure SetElliptic(Value: Boolean);
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure CalcActiveGlyph;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure Click; override;
  published
    { Published declarations }
    property Glyph: TBitmap
      read FGlyph
      write SetGlyph;
    property Caption: TCaption
      read FCaption
      write SetCaption;
    property DrawFrame: boolean
      read FDrawFrame
      write SetDrawFrame
      default False;
    property FrameColor: TColor
      read FFrameColor
      write SetFrameColor
      default clBtnShadow;
    property NumGlyphs: TNumGlyphs
      read GetNumGlyphs
      write SetNumGlyphs
      default 1;
    property GlyphLayout: TGlyphLayout
      read FGlyphLayout
      write SetGlyphLayout
      default glGlyphLeft;
    property ButtonOrientation: TButtonOrientation
      read FButtonOrientation
      write SetButtonOrientation
      default boHorizontal;
    property A3DTopCl: TColor
      read F3DTopCl
      write Set3DTopCl;
    property A3DBotCl: TColor
      read F3DBotCl
      write Set3DBotCl;
    property Elliptic: Boolean
      read FElliptic
      write SetElliptic;
    property Enabled;
    property Font;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('siComponents', [TsiFlatBtn]);
end;

constructor TsiFlatBtn.Create(AOwner: TComponent);
begin
  inherited;
  FGlyph := TBitmap.Create;
  BMP := TBitmap.Create;
  ActiveGlyph := TBitmap.Create;
  FIsDown := False;
  FHasMouse := False;
  Height := 25;
  Width := 75;
  FDrawFrame := False;
  FFrameColor := clBtnShadow;
  FGlyphNums := 1;
  FGlyphLayout := glGlyphLeft;
  F3DTopCl := clBtnHighLight;
  F3DBotCl := clBtnShadow;
  FElliptic := True;
end;

destructor TsiFlatBtn.Destroy;
begin
  FGlyph.Free;
  BMP.Free;
  ActiveGlyph.Free;
  inherited;
end;

procedure TsiFlatBtn.SetGlyph(Value: TBitmap);
begin
  FGlyph.Assign(Value);
  if (Value <> nil) and (Value.Height > 0) then
  begin
    if Value.Width mod Value.Height = 0 then
    begin
      FGlyphNums := Value.Width div Value.Height;
      if FGlyphNums > 4 then FGlyphNums := 1;
    end
    else FGlyphNums := 1;
  end;
  Invalidate;
end;

procedure TsiFlatBtn.SetCaption(Value: TCaption);
begin
  if FCaption <> Value then begin
    FCaption := Value;
    Invalidate;
  end;
end;

procedure TsiFlatBtn.SetDrawFrame(Value: Boolean);
begin
  if FDrawFrame <> Value then begin
    FDrawFrame := Value;
    Invalidate;
  end;
end;

procedure TsiFlatBtn.SetFrameColor(Value: TColor);
begin
  if FFrameColor <> Value then begin
    FFrameColor := Value;
    Invalidate;
  end;
end;

function TsiFlatBtn.GetNumGlyphs: TNumGlyphs;
begin
  Result := FGlyphNums;
end;

procedure TsiFlatBtn.SetNumGlyphs(Value: TNumGlyphs);
begin
  if Value < 0 then Value := 1
  else if Value > 4 then Value := 4;
  if Value <> FGlyphNums then
  begin
    FGlyphNums := Value;
    Invalidate;
  end;
end;

procedure TsiFlatBtn.CalcActiveGlyph;
begin
  if FGlyphNums = 1 then begin
    if ActiveGlyph = nil then ActiveGlyph := TBitmap.Create;
    ActiveGlyph.Assign(FGlyph);
    Exit;
  end;
  ActiveGlyph.Free;
  ActiveGlyph := nil;
  if (not Enabled) then begin
    if (FGlyphNums >= 2) then begin
      ActiveGlyph := TBitmap.Create;
      ActiveGlyph.Width := FGlyph.Width div FGlyphNums;
      ActiveGlyph.Height := FGlyph.Height;
      ActiveGlyph.Canvas.CopyRect(Rect(0, 0, ActiveGlyph.Width, ActiveGlyph.Height), FGlyph.Canvas, Rect(ActiveGlyph.Width, 0, 2 * ActiveGlyph.Width, FGlyph.Height))
    end;
    Exit;
  end;
  if not FHasMouse then begin
    ActiveGlyph := TBitmap.Create;
    ActiveGlyph.Width := FGlyph.Width div FGlyphNums;
    ActiveGlyph.Height := FGlyph.Height;
    ActiveGlyph.Canvas.CopyRect(Rect(0, 0, ActiveGlyph.Width, ActiveGlyph.Height), FGlyph.Canvas, Rect(0, 0, ActiveGlyph.Width, FGlyph.Height));
    Exit;
  end;
  if (FIsDown) and (FGlyphNums = 4) then begin
    ActiveGlyph := TBitmap.Create;
    ActiveGlyph.Width := FGlyph.Width div FGlyphNums;
    ActiveGlyph.Height := FGlyph.Height;
    ActiveGlyph.Canvas.CopyRect(Rect(0, 0, ActiveGlyph.Width, ActiveGlyph.Height), FGlyph.Canvas, Rect(ActiveGlyph.Width * 3, 0, ActiveGlyph.Width * 4, FGlyph.Height));
    Exit;
  end;
  if (not FIsDown) and (FGlyphNums >= 3) then begin
    ActiveGlyph := TBitmap.Create;
    ActiveGlyph.Width := FGlyph.Width div FGlyphNums;
    ActiveGlyph.Height := FGlyph.Height;
    ActiveGlyph.Canvas.CopyRect(Rect(0, 0, ActiveGlyph.Width, ActiveGlyph.Height), FGlyph.Canvas, Rect(ActiveGlyph.Width * 2, 0, ActiveGlyph.Width * 3, FGlyph.Height));
    Exit;
  end;
end;

procedure TsiFlatBtn.SetGlyphLayout(Value: TGlyphLayout);
begin
  if Value <> FGlyphLayout then begin
    FGlyphLayout := Value;
    Invalidate;
  end;
end;

procedure TsiFlatBtn.SetButtonOrientation(Value: TButtonOrientation);
begin
  if Value <> FButtonOrientation then begin
    FButtonOrientation := Value;
    Invalidate;
  end;
end;

procedure TsiFlatBtn.DrawRaised;
begin
  with BMP.Canvas do begin
    Font.Assign(Self.Font);
    Brush.Color := TForm(GetOwner).Color;
    Brush.Style := bsSolid;
    FillRect(FMyRect);
    Brush.Style := bsClear;
    if Enabled then DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, FTextFlag)
    else begin
      Brush.Style := bsClear;
      Font.Color := clBtnHighlight;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, FTextFlag);
      OffsetRect(TextBounds, -1, -1);
      Font.Color := clBtnShadow;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, FTextFlag);
    end;
    Pen.Style := psSolid;
    Pen.Color := F3DTopCl;
    if FElliptic then RoundRect(FMyRect.Left, FMyRect.Top, FMyRect.Right, FMyRect.Bottom, FRadius, FRadius)
    else Rectangle(FMyRect.Left+1, FMyRect.Top, FMyRect.Right, FMyRect.Bottom);
    Pen.Style := psSolid;
    Pen.Color := F3DBotCl;
    if FElliptic then RoundRect(FMyRect.Left+1, FMyRect.Top+1, FMyRect.Right, FMyRect.Bottom, FRadius, FRadius)
    else Rectangle(FMyRect.Left+2, FMyRect.Top+1, FMyRect.Right, FMyRect.Bottom);
    Pen.Style := psSolid;
    Pen.Color := clBtnFace;
    if FElliptic then RoundRect(FMyRect.Left+1, FMyRect.Top+1, FMyRect.Right-1, FMyRect.Bottom-1, FRadius, FRadius)
    else Rectangle(FMyRect.Left+2, FMyRect.Top+1, FMyRect.Right-1, FMyRect.Bottom-1);
    BMP.Transparent := True;
    if ActiveGlyph <> nil then begin
      ActiveGlyph.Transparent := True;
      if not ActiveGlyph.Empty then
        Draw(FGlyphLeft, FGlyphTop, ActiveGlyph);
    end;
  end;
end;

procedure TsiFlatBtn.DrawLowered;
begin
  with BMP.Canvas do begin
    Font.Assign(Self.Font);
    Brush.Color := TForm(GetOwner).Color;
    Brush.Style := bsSolid;
    FillRect(FMyRect);
    Inc(TextBounds.Left,2);
    Inc(TextBounds.Top,2);
    DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, FTextFlag);
    Brush.Style := bsClear;
    Pen.Style := psSolid;
    Pen.Color := F3DBotCl;
    if FElliptic then RoundRect(FMyRect.Left, FMyRect.Top, FMyRect.Right, FMyRect.Bottom, FRadius, FRadius)
    else Rectangle(FMyRect.Left+1, FMyRect.Top, FMyRect.Right, FMyRect.Bottom);
    Pen.Style := psSolid;
    Pen.Color := F3DTopCl;
    if FElliptic then RoundRect(FMyRect.Left+1, FMyRect.Top+1, FMyRect.Right, FMyRect.Bottom, FRadius, FRadius)
    else Rectangle(FMyRect.Left+2, FMyRect.Top+1, FMyRect.Right, FMyRect.Bottom);
    Pen.Style := psSolid;
    Pen.Color := clBtnFace;
    if FElliptic then RoundRect(FMyRect.Left+1, FMyRect.Top+1, FMyRect.Right-1, FMyRect.Bottom-1, FRadius, FRadius)
    else Rectangle(FMyRect.Left+2, FMyRect.Top+1, FMyRect.Right-1, FMyRect.Bottom-1);
    BMP.Transparent := True;
    if ActiveGlyph <> nil then begin
      ActiveGlyph.Transparent := True;
      if not ActiveGlyph.Empty then
        Draw(FGlyphLeft + 2, FGlyphTop + 2, ActiveGlyph);
    end;
  end;
end;

procedure TsiFlatBtn.DrawUsual;
begin
  with BMP.Canvas do begin
    Font.Assign(Self.Font);
    Brush.Color := TForm(GetOwner).Color;
    Brush.Style := bsSolid;
    FillRect(FMyRect);
    if Enabled then DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, FTextFlag)
    else begin
      Brush.Style := bsClear;
      Font.Color := clBtnHighlight;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, FTextFlag);
      OffsetRect(TextBounds, -1, -1);
      Font.Color := clBtnShadow;
      DrawText(Handle, PChar(Caption), Length(Caption), TextBounds, FTextFlag);
    end;
    Brush.Style := bsClear;
    if FDrawFrame then begin
      Pen.Style := psSolid;
      Pen.Color := FFrameColor;
      if FElliptic then RoundRect(FMyRect.Left, FMyRect.Top, FMyRect.Right, FMyRect.Bottom, FRadius, FRadius)
      else Rectangle(FMyRect.Left+2, FMyRect.Top, FMyRect.Right, FMyRect.Bottom);
    end;
    BMP.Transparent := True;
    if ActiveGlyph <> nil then begin
      ActiveGlyph.Transparent := True;
      if not ActiveGlyph.Empty then
        Draw(FGlyphLeft, FGlyphTop, ActiveGlyph);
    end;
  end;
end;

procedure TsiFlatBtn.Paint;
begin
  BMP.Width := Width;
  BMP.Height := Height;
  FMyRect := ClientRect;
  if FButtonOrientation = boHorizontal then
    FRadius := FMyRect.Bottom - FMyRect.Top
  else
    FRadius := FMyRect.Right - FMyRect.Left;
  CalcActiveGlyph;
  Canvas.Font.Assign(Font);
    case FGlyphLayout of
      glGlyphLeft: begin
        FTextFlag := DT_CENTER or DT_VCENTER or DT_SINGLELINE;
        TextBounds := Rect(FRadius div 4 + 5, 0, Width, Height);
        FGlyphLeft := Width div 8;
        if ActiveGlyph <> nil then begin
          if Canvas.TextWidth(Caption) > Width - ActiveGlyph.Width - FGlyphLeft - 2 then begin
            FTextFlag := DT_LEFT or DT_VCENTER or DT_SINGLELINE;
            TextBounds := Rect(Width div 8 + ActiveGlyph.Width + 2, 0, Width, Height);
          end;
          FGlyphTop := Height div 2 - ActiveGlyph.Height div 2;
        end;
      end;
      glGlyphRight: begin
        FTextFlag := DT_CENTER or DT_VCENTER or DT_SINGLELINE;
        FGlyphLeft := Width - Width div 4;
        TextBounds := Rect(0, 0, FGlyphLeft + FRadius div 4, Height);
        if ActiveGlyph <> nil then begin
          FGlyphTop := Height div 2 - ActiveGlyph.Height div 2;
          if Canvas.TextWidth(Caption) > FGlyphLeft - 2 then begin
            FTextFlag := DT_RIGHT or DT_VCENTER or DT_SINGLELINE;
            TextBounds := Rect(0, 0, FGlyphLeft - 2, Height);
          end;
        end;
      end;
      glGlyphTop: begin
        FTextFlag := DT_CENTER or DT_VCENTER or DT_SINGLELINE;
        TextBounds := Rect(0 , 0, Width, Height);
        if ActiveGlyph <> nil then begin
          FGlyphLeft := Width div 2 - ActiveGlyph.Width div 2;
          FGlyphTop := Height div 8;
        end;
      end;
      glGlyphBottom: begin
        FTextFlag := DT_CENTER or DT_VCENTER or DT_SINGLELINE;
        TextBounds := Rect(0, 0, Width, Height);
        if ActiveGlyph <> nil then begin
          FGlyphLeft := Width div 2 - ActiveGlyph.Width div 2;
          FGlyphTop := Height - Height div 4;
        end;
      end;
    end; {case...}
  if not (csDesigning in ComponentState) then begin
    if ((not FIsDown) and (not FHasMouse)) or (not Enabled) then
      DrawUsual
    else
      if FIsDown then DrawLowered
      else
        DrawRaised;
  end
  else DrawRaised;
  Canvas.Draw(0, 0, BMP);
end;

procedure TsiFlatBtn.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not Enabled then Exit;
  FHasMouse := True;
  FIsDown := False;
  Invalidate;
end;

procedure TsiFlatBtn.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if not Enabled then Exit;
  FHasMouse := False;
  FIsDown := False;
  Invalidate;
end;

procedure TsiFlatBtn.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  if Button = mbLeft then begin
    FIsDown := True;
    FHasMouse := True;
    Invalidate;
  end;
end;

procedure TsiFlatBtn.Click;
begin
  FIsDown := False;
//  FHasMouse := False;
//  Invalidate;
  inherited;
end;

procedure TsiFlatBtn.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  if Button = mbLeft then begin
    FIsDown := False;
    if (X >= 0) and (Y >= 0) and (X <= FMyRect.Right) and (Y <= FMyRect.Bottom) and FHasMouse then begin
      Invalidate;
    end;
  end;
end;

procedure TsiFlatBtn.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and Enabled then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure TsiFlatBtn.Set3DTopCl(Value: TColor);
begin
  if F3DTopCl <> Value then begin
    F3DTopCl := Value;
    Invalidate;
  end;
end;

procedure TsiFlatBtn.Set3DBotCl(Value: TColor);
begin
  if F3DBotCl <> Value then begin
    F3DBotCl := Value;
    Invalidate;
  end;
end;

procedure TsiFlatBtn.SetElliptic(Value: Boolean);
begin
  if Value <> FElliptic then begin
    FElliptic := Value;
    Invalidate;
  end;
end;

end.
